@@ -1,17 +1,18 @@
-from typing import TypedDict, Union, Sequence, Optional, Generator, List
+import sys
+from typing import TypedDict, Union, Sequence, List
 from copy import deepcopy
 from pathlib import Path
+import random
 
 import numpy as np
 import torch
 import torch.nn as nn
 import torchvision.transforms as TF
-from torch import randperm, default_generator
 from torch._utils import _accumulate
 from torch.utils.data import Subset
 from torch.utils.data.dataset import Dataset
 
-from vp_suite.utils.utils import set_from_kwarg, get_public_attrs
+from vp_suite.utils.utils import set_from_kwarg, get_public_attrs, PytestExpectedException
 
 
 CROPS = [TF.CenterCrop, TF.RandomCrop]
@@ -28,6 +29,7 @@ class VPData(TypedDict):
     """
     frames: torch.Tensor  #: Video frames: torch tensors of shape [t, c, h, w].
     actions: torch.Tensor  #: Actions per frame: torch tensors of shape [t, a].
+    origin: str  #: A string specifying the source of the data.
 
 
 class VPSubset(Subset):
@@ -48,13 +50,13 @@ class VPDataset(Dataset):
         In order to fully prepare the dataset, :meth:`self.set_seq_len()` has to be called with the desired amount
         of frames and the seq_step. Afterwards, the VPDataset object. is ready to be queried for data.
     """
-    NON_CONFIG_VARS = ["functions", "VALID_SPLITS", "NON_CONFIG_VARS", "ready_for_usage",
-                       "total_frames", "seq_len", "frame_offsets"]  #: Variables that do not get included in the dict returned by :meth:`self.config()`.
+    NON_CONFIG_VARS = ["functions",  "ready_for_usage", "total_frames", "seq_len", "frame_offsets", "data_dir"]  #: Variables that do not get included in the dict returned by :meth:`self.config()` (Constants are not included either).
 
     # DATASET CONSTANTS
     NAME: str = NotImplemented  #: The dataset's name.
     REFERENCE: str = None  #: The reference (publication) where the original dataset is introduced.
     IS_DOWNLOADABLE: str = None  #: A string identifying whether the dataset can be (freely) downloaded.
+    ON_THE_FLY: bool = False  #: If true, accessing the dataset means data is generated on the fly rather than fetched from storage.
     DEFAULT_DATA_DIR: Path = NotImplemented  #: The default save location of the dataset files.
     VALID_SPLITS = ["train", "test"]  #: The valid arguments for specifying splits.
     MIN_SEQ_LEN: int = NotImplemented  #: The minimum sequence length provided by the dataset.
@@ -64,6 +66,7 @@ class VPDataset(Dataset):
     # dataset hyper-parameters
     img_shape: (int, int, int) = NotImplemented  #: Shape of a single frame as returned by `__getitem()__`.
     train_to_val_ratio: float = 0.8  #: The ratio of files that will be training data (rest will be validation data). For bigger datasets, this ratio can be set closer to 1.
+    train_val_seed = 1234  #: Random seed used to separate training and validation data.
     transform: nn.Module = None  #: This module gets called in the preprocessing step and consists of pre-specified cropping, resizing and augmentation layers.
     split: str = None  #: The dataset's split identifier (i.e. whether it's a training/validation/test dataset).
     seq_step: int = 1  #: With a step N, every Nth frame is included in the returned sequence.
@@ -91,9 +94,13 @@ class VPDataset(Dataset):
         self.data_dir = dataset_kwargs.get("data_dir", self.data_dir)
         if self.data_dir is None:
             if not self.default_available(self.split, **dataset_kwargs):
-                print(f"downloading/preparing dataset '{self.NAME}' "
-                      f"and saving it to '{self.DEFAULT_DATA_DIR}'...")
-                self.download_and_prepare_dataset()
+                if "pytest" in sys.modules:  # don't download datasets if running this code from the test suite
+                    raise PytestExpectedException(f"Default for Dataset '{self.NAME}' is unavailable "
+                                                  f"and pytest won't download it")
+                else:
+                    print(f"downloading/preparing dataset '{self.NAME}' "
+                          f"and saving it to '{self.DEFAULT_DATA_DIR}'...")
+                    self.download_and_prepare_dataset()
             self.data_dir = str(self.DEFAULT_DATA_DIR.resolve())
 
         # DATA PREPROCESSING: convert -> permute -> scale -> crop -> resize -> augment
@@ -150,8 +157,8 @@ class VPDataset(Dataset):
             "img_w": img_w,
             "img_c": img_c,
             "action_size": self.ACTION_SIZE,
-            "supports_actions": self.ACTION_SIZE > 0,
             "tensor_value_range": [self.value_range_min, self.value_range_max],
+            "NAME": self.NAME
         }
         return {**attr_dict, **extra_config}
 
@@ -183,6 +190,10 @@ class VPDataset(Dataset):
         r""" Optional dataset-specific logic for :meth:`self.set_seq_len()`. """
         pass
 
+    def reset_rng(self):
+        r""" Optional logic for resetting the RNG of a dataset. """
+        pass
+
     def __len__(self) -> int:
         r"""
         Returns: The number of available data points of this dataset (its "size").
@@ -226,7 +237,7 @@ class VPDataset(Dataset):
             elif x.dtype == np.uint8:
                 x = x.astype(np.float32) / ((1 << 8) - 1)
             elif x.dtype == float:
-                pass
+                x = x / ((1 << 8) - 1)
             else:
                 raise ValueError(f"if providing numpy arrays, only dtypes "
                                  f"np.uint8, np.float and np.uint16 are supported (given: {x.dtype})")
@@ -340,7 +351,7 @@ class VPDataset(Dataset):
             len_main = len(D_main)
             len_train = int(len_main * cls.train_to_val_ratio)
             len_val = len_main - len_train
-            D_train, D_val = _random_split(D_main, [len_train, len_val])
+            D_train, D_val = _random_split(D_main, [len_train, len_val], cls.train_val_seed)
         else:
             D_train = cls("train", **dataset_kwargs)
             D_val = cls("val", **dataset_kwargs)
@@ -363,8 +374,7 @@ class VPDataset(Dataset):
         return D_test
 
 
-def _random_split(dataset: VPDataset, lengths: Sequence[int],
-                  generator: Optional[Generator] = default_generator) -> List[VPSubset]:
+def _random_split(dataset: VPDataset, lengths: Sequence[int], random_seed: int) -> List[VPSubset]:
     r"""
     Custom implementation of torch.utils.data.random_split that returns SubsetWrappers.
 
@@ -374,13 +384,17 @@ def _random_split(dataset: VPDataset, lengths: Sequence[int],
     Args:
         dataset (Dataset): Dataset to be split.
         lengths (sequence): lengths of splits to be produced.
-        generator (Generator): Generator used for the random permutation.
+        random_seed (int): RNG seed used for the random permutation.
+
+    Returns:
+        A list of VPSubsets containing the randomly split datasets.
     """
 
     # Cannot verify that dataset is Sized
     if sum(lengths) != len(dataset):
         raise ValueError("Sum of input lengths does not equal the length of the input dataset!")
 
-    indices = randperm(sum(lengths), generator=generator).tolist()
+    indices = list(range(sum(lengths)))
+    random.Random(random_seed).shuffle(indices)
     return [VPSubset(dataset, indices[offset - length: offset])
             for offset, length in zip(_accumulate(lengths), lengths)]
