@@ -5,17 +5,17 @@ from torch.optim.optimizer import Optimizer
 from tqdm import tqdm
 from vp_suite.utils.utils import set_from_kwarg, get_public_attrs
 from vp_suite.measure.loss_provider import PredictionLossProvider
-from vp_suite.base.base_dataset import VPData
+from vp_suite.base import VPData
 
 
-class VideoPredictionModel(nn.Module):
+class VPModel(nn.Module):
     r"""
     The base class for all video prediction models. Each model ought to provide two forward pass/prediction methods
     (the default :meth:`self.forward()` method and :meth:`pred_1()`, which predicts a single frame) as well as two
     utility methods (:meth:`train_iter()` for a single training epoch on a given dataset loader and, analogously,
     :meth:`eval_iter()` for a single epoch of validation iteration).
     """
-    NON_CONFIG_VARS = ["functions", "REQUIRED_ARGS"]  #: Variables that do not get included in the dict returned by :meth:`self.config()`.
+    NON_CONFIG_VARS = ["functions", "model_dir", "dump_patches", "training"]  #: Variables that do not get included in the dict returned by :meth:`self.config()` (Constants are not included either).
 
     # MODEL CONSTANTS
     NAME = None  #: The model's name.
@@ -44,7 +44,7 @@ class VideoPredictionModel(nn.Module):
             device (str): The device identifier for the module.
             **model_kwargs (Any): Model arguments such as hyperparameters, input shapes etc.
         """
-        super(VideoPredictionModel, self).__init__()
+        super(VPModel, self).__init__()
 
         # set required parameters
         self.device = device
@@ -80,10 +80,11 @@ class VideoPredictionModel(nn.Module):
             "img_h": img_h,
             "img_w": img_w,
             "img_c": img_c,
+            "NAME": self.NAME
         }
         return {**attr_dict, **extra_config}
 
-    def unpack_data(self, data: VPData, config: dict, reverse: bool = False):
+    def unpack_data(self, data: VPData, config: dict, reverse: bool = False, complete: bool = False):
         r"""
         Extracts inputs and targets from a data blob.
 
@@ -91,17 +92,25 @@ class VideoPredictionModel(nn.Module):
             data (VPData): The given VPData data blob/dictionary containing frames and actions.
             config (dict): The current run configuration specifying how to extract the data from the given data blob.
             reverse (bool): If specified, reverses the input first
+            complete (bool): If specified, input_frames will also contain the to-be-predicted frames (just like with NEEDS_COMPLETE_INPUT)
 
         Returns: The specified amount of input/target frames as well as the actions. All inputs will come in the
         shape the model expects as input later.
         """
         img_data = data["frames"].to(config["device"])  # [b, T, c, h, w], with T = total_frames
-        actions = data["actions"].to( config["device"])  # [b, T-1, a]. Action t happens between frame t and t+1
+        actions = data["actions"].to(config["device"])  # [b, T-1, a]. Action t happens between frame t and t+1
+        if img_data.ndim == 4:  # prepend batch dimension if not given
+            img_data = img_data.unsqueeze(0)
+            actions = actions.unsqueeze(0)
         if reverse:
-            img_data = img_data[:, ::-1]
-            actions = actions[:, ::-1]
-        input_frames = img_data if self.NEEDS_COMPLETE_INPUT else img_data[:, :config["context_frames"]]
-        target_frames = img_data[:, config["context_frames"]: config["context_frames"] + config["pred_frames"]]
+            img_data = torch.flip(img_data, dims=[1])
+            actions = torch.flip(actions, dims=[1])
+        T_in, T_pred = config["context_frames"], config["pred_frames"]
+        if self.NEEDS_COMPLETE_INPUT or complete:
+            input_frames = img_data[:, :T_in+T_pred]
+            target_frames = input_frames[:, T_in:].clone()
+        else:
+            input_frames, target_frames = torch.split(img_data[:, :T_in+T_pred], [T_in, T_pred], dim=1)
         return input_frames, target_frames, actions
 
     def pred_1(self, x: torch.Tensor, **kwargs):
@@ -113,7 +122,6 @@ class VideoPredictionModel(nn.Module):
             **kwargs (Any): Optional input parameters such as actions.
 
         Returns: A single frame as a tensor of shape [b, c, h, w].
-
         """
         raise NotImplementedError
 
@@ -127,7 +135,6 @@ class VideoPredictionModel(nn.Module):
             **kwargs ():
 
         Returns: A batch of sequences of `p` predicted frames as a tensor of shape [b, p, c, h, w].
-
         """
         predictions = []
         for i in range(pred_frames):
@@ -183,7 +190,6 @@ class VideoPredictionModel(nn.Module):
 
         Returns: A dictionary containing the averages value for each loss type specified for usage,
         as well as the value for the 'indicator' loss (the loss used for determining overall model improvement).
-
         """
         self.eval()
         loop = tqdm(loader)
