@@ -7,8 +7,8 @@ from tqdm import tqdm
 from tfrecord.tools.tfrecord2idx import create_index
 from tfrecord.torch.dataset import TFRecordDataset
 
-from vp_suite.base.base_dataset import VPDataset, VPData
-import vp_suite.constants as constants
+from vp_suite.base import VPDataset, VPData
+from vp_suite.defaults import SETTINGS
 
 class BAIRPushingDataset(VPDataset):
     r"""
@@ -22,7 +22,7 @@ class BAIRPushingDataset(VPDataset):
     NAME = "BAIR robot pushing"
     REFERENCE = "https://arxiv.org/abs/1710.05268"
     IS_DOWNLOADABLE = "Yes"
-    DEFAULT_DATA_DIR = constants.DATA_PATH / "bair_robot_pushing"
+    DEFAULT_DATA_DIR = SETTINGS.DATA_PATH / "bair_robot_pushing"
     MIN_SEQ_LEN = 30
     ACTION_SIZE = 4
     DATASET_FRAME_SHAPE = (64, 64, 3)
@@ -52,14 +52,15 @@ class BAIRPushingDataset(VPDataset):
         if not self.ready_for_usage:
             raise RuntimeError("Dataset is not yet ready for usage (maybe you forgot to call set_seq_len()).")
 
-        rgb_raw = np.load(self.obs_fps[i])
+        obs_fp = self.obs_fps[i]
+        rgb_raw = np.load(obs_fp)
         rgb_raw = rgb_raw[:self.seq_len:self.seq_step]
         rgb = self.preprocess(rgb_raw)  # [t, c, h, w]
 
         actions = torch.from_numpy(np.load(self.actions_fps[i])).float()
         actions = actions[:self.seq_len:self.seq_step]  # [t, a]
 
-        data = { "frames": rgb, "actions": actions }
+        data = {"frames": rgb, "actions": actions, "origin": obs_fp}
         return data
 
     @classmethod
@@ -105,9 +106,6 @@ def split_bair_traj_files(data_dir: Path, delete_tfrecords: bool):
     Args:
         data_dir (Path): Specified path to dataset.
         delete_tfrecords (bool): If True, delete the .tfrecord files after extraction.
-
-    Returns:
-
     """
     bair_ep_length = 30
 
