@@ -6,8 +6,8 @@ import cv2
 import torch
 from tqdm import tqdm
 
-from vp_suite.base.base_dataset import VPDataset, VPData
-import vp_suite.constants as constants
+from vp_suite.base import VPDataset, VPData
+from vp_suite.defaults import SETTINGS
 from vp_suite.utils.utils import set_from_kwarg, read_video
 
 
@@ -22,7 +22,7 @@ class CaltechPedestrianDataset(VPDataset):
     NAME = "Caltech Pedestrian"
     REFERENCE = "http://www.vision.caltech.edu/Image_Datasets/CaltechPedestrians/"
     IS_DOWNLOADABLE = "Yes"
-    DEFAULT_DATA_DIR = constants.DATA_PATH / "caltech_pedestrian"
+    DEFAULT_DATA_DIR = SETTINGS.DATA_PATH / "caltech_pedestrian"
     VALID_SPLITS = ["train", "val", "test"]
     MIN_SEQ_LEN = 568  #: Minimum number of frames across all sequences (1322 in 2nd-shortest, 2175 in longest)
     ACTION_SIZE = 0
@@ -32,7 +32,6 @@ class CaltechPedestrianDataset(VPDataset):
     TEST_SETS = [f"set{i:02d}" for i in range(6, 11)]  #: The official test sets.
 
     train_to_val_ratio = 0.9
-    train_val_seed = 1234  #: The random seed used to separate training and validation data.
 
     def __init__(self, split, **dataset_kwargs):
         super(CaltechPedestrianDataset, self).__init__(split, **dataset_kwargs)
@@ -82,7 +81,7 @@ class CaltechPedestrianDataset(VPDataset):
         vid = self.preprocess(vid)  # [t, c, h, w]
         actions = torch.zeros((self.total_frames, 1))  # [t, a], actions should be disregarded in training logic
 
-        data = {"frames": vid, "actions": actions}
+        data = {"frames": vid, "actions": actions, "origin": f"{sequence_path}, start frame: {start_idx}"}
         return data
 
     def __len__(self):
@@ -99,8 +98,8 @@ class CaltechPedestrianDataset(VPDataset):
             print(f"Found sequence data in {str(d_path.resolve())} -> Won't download {cls.NAME}")
         except StopIteration:
             from vp_suite.utils.utils import run_shell_command
-            import vp_suite.constants as constants
-            prep_script = (constants.PKG_RESOURCES / 'get_dataset_caltech_pedestrian.sh').resolve()
+            from vp_suite.defaults import SETTINGS
+            prep_script = (SETTINGS.PKG_RESOURCES / 'get_dataset_caltech_pedestrian.sh').resolve()
             run_shell_command(f"{prep_script} {cls.DEFAULT_DATA_DIR}")
 
         # pre-count frames of all sequences if not yet done so (makes data fetching faster later on)
