@@ -6,8 +6,8 @@ from pathlib import Path
 import torch
 from tqdm import tqdm
 
-from vp_suite.base.base_dataset import VPDataset, VPData
-import vp_suite.constants as constants
+from vp_suite.base import VPDataset, VPData
+from vp_suite.defaults import SETTINGS
 from vp_suite.utils.utils import set_from_kwarg, get_frame_count, read_video
 
 
@@ -23,7 +23,7 @@ class Human36MDataset(VPDataset):
     NAME = "Human 3.6M"
     REFERENCE = "http://vision.imar.ro/human3.6m/description.php"
     IS_DOWNLOADABLE = "With Registered Account"
-    DEFAULT_DATA_DIR = constants.DATA_PATH / "human36m"
+    DEFAULT_DATA_DIR = SETTINGS.DATA_PATH / "human36m"
     VALID_SPLITS = ["train", "val", "test"]
     MIN_SEQ_LEN = 994  #: Minimum number of frames across all sequences (6349 in longest).
     ACTION_SIZE = 0
@@ -35,7 +35,6 @@ class Human36MDataset(VPDataset):
                      'Waiting', 'WalkDog', 'WalkTogether', 'Walking', 'WalkingDog']  #: All recorded scenarios of the dataset.
 
     train_to_val_ratio = 0.96
-    train_val_seed = 1234  #: Random seed used to separate training and validation data.
     scenarios = None  #: Scenarios chosen for the current dataset instance (defaults to `self.ALL_SCENARIOS`)
 
     def __init__(self, split, **dataset_kwargs):
@@ -79,14 +78,14 @@ class Human36MDataset(VPDataset):
                 self.sequences_with_frame_index.append((vfp, idx))
 
     def __getitem__(self, i) -> VPData:
-        vid_fp, start_idx = self.sequences_with_frame_index[i]
-        vid = read_video(vid_fp, img_size=self.img_shape[1:],
+        sequence_path, start_idx = self.sequences_with_frame_index[i]
+        vid = read_video(sequence_path, img_size=self.img_shape[1:],
                          start_index=start_idx, num_frames=self.seq_len)  # [T, h, w, c]
         vid = vid[::self.seq_step]  # [t, h, w, c]
         vid = self.preprocess(vid)  # [t, c, h, w]
         actions = torch.zeros((self.total_frames, 1))  # [t, a], actions should be disregarded in training logic
 
-        data = {"frames": vid, "actions": actions}
+        data = {"frames": vid, "actions": actions, "origin": f"{sequence_path}, start frame: {start_idx}"}
         return data
 
     def __len__(self):
@@ -99,9 +98,9 @@ class Human36MDataset(VPDataset):
         vid_filepaths: [Path] = list(d_path.rglob(f"**/*.mp4"))
         if len(vid_filepaths) == 0:  # no data available -> download data and unpack
             from vp_suite.utils.utils import run_shell_command
-            import vp_suite.constants as constants
+            from vp_suite.defaults import SETTINGS
             print(f"Downloading and extracting {cls.NAME} - Videos...")
-            prep_script = (constants.PKG_RESOURCES / 'get_dataset_human36m.sh').resolve()
+            prep_script = (SETTINGS.PKG_RESOURCES / 'get_dataset_human36m.sh').resolve()
             run_shell_command(f"{prep_script} {cls.DEFAULT_DATA_DIR}")
 
         # open all videos to get their frame counts (speeds up dataset creation later)
