@@ -5,8 +5,8 @@ import cv2
 import numpy as np
 import torch
 
-from vp_suite.base.base_dataset import VPDataset, VPData
-import vp_suite.constants as constants
+from vp_suite.base import VPDataset, VPData
+from vp_suite.defaults import SETTINGS
 from vp_suite.utils.utils import set_from_kwarg
 
 
@@ -20,7 +20,7 @@ class KITTIRawDataset(VPDataset):
     NAME = "KITTI raw"
     REFERENCE = "http://www.cvlibs.net/datasets/kitti/raw_data.php"
     IS_DOWNLOADABLE = "With Registered Account"
-    DEFAULT_DATA_DIR = constants.DATA_PATH / "kitti_raw"
+    DEFAULT_DATA_DIR = SETTINGS.DATA_PATH / "kitti_raw"
     VALID_SPLITS = ["train", "val", "test"]
     MIN_SEQ_LEN = 994  #: Minimum number of frames across all sequences (6349 in longest).
     ACTION_SIZE = 0
@@ -32,7 +32,6 @@ class KITTIRawDataset(VPDataset):
     trainval_to_test_ratio = 0.8  #: The ratio of files that will be training/validation data (rest will be test data).
     train_to_val_ratio = 0.9
     trainval_test_seed = 1234  #: The random seed used to separate training/validation and testing data.
-    train_val_seed = 1234  #: The random seed used to separate training and validation data.
 
     def __init__(self, split, **dataset_kwargs):
         super(KITTIRawDataset, self).__init__(split, **dataset_kwargs)
@@ -55,12 +54,13 @@ class KITTIRawDataset(VPDataset):
 
         # slice accordingly
         slice_idx = max(1, int(len(sequence_dirs) * self.trainval_to_test_ratio))
-        random.Random(self.train_val_seed).shuffle(sequence_dirs)
+        random.Random(self.trainval_test_seed).shuffle(sequence_dirs)
         if self.split == "test":
             sequence_dirs = sequence_dirs[slice_idx:]
         else:
             sequence_dirs = sequence_dirs[:slice_idx]
             slice_idx = max(1, int(len(sequence_dirs) * self.train_to_val_ratio))
+            random.Random(self.train_val_seed).shuffle(sequence_dirs)
             if self.split == "train":
                 sequence_dirs = sequence_dirs[:slice_idx]
             else:
@@ -91,7 +91,7 @@ class KITTIRawDataset(VPDataset):
         vid = self.preprocess(vid)  # [t, *self.img_shape]
         actions = torch.zeros((self.total_frames, 1))  # [t, a], actions should be disregarded in training logic
 
-        data = {"frames": vid, "actions": actions}
+        data = {"frames": vid, "actions": actions, "origin": f"{sequence_path}, start frame: {start_idx}"}
         return data
 
     def __len__(self):
@@ -108,6 +108,6 @@ class KITTIRawDataset(VPDataset):
             print(f"Found image data in {str(d_path.resolve())} -> Won't download {cls.NAME}")
         except StopIteration:
             from vp_suite.utils.utils import run_shell_command
-            import vp_suite.constants as constants
-            prep_script = (constants.PKG_RESOURCES / 'get_dataset_kitti_raw.sh').resolve()
+            from vp_suite.defaults import SETTINGS
+            prep_script = (SETTINGS.PKG_RESOURCES / 'get_dataset_kitti_raw.sh').resolve()
             run_shell_command(f"{prep_script} {cls.DEFAULT_DATA_DIR}")
