@@ -6,8 +6,8 @@ import imageio
 import torchfile
 from pathlib import Path
 
-from vp_suite.base.base_dataset import VPDataset, VPData
-import vp_suite.constants as constants
+from vp_suite.base import VPDataset, VPData
+from vp_suite.defaults import SETTINGS
 
 class KTHActionsDataset(VPDataset):
     r"""
@@ -27,13 +27,15 @@ class KTHActionsDataset(VPDataset):
     NAME = "KTH Actions"
     REFERENCE = "https://doi.org/10.1109/ICPR.2004.1334462"
     IS_DOWNLOADABLE = "Yes"
-    DEFAULT_DATA_DIR = constants.DATA_PATH / "kth_actions"
+    DEFAULT_DATA_DIR = SETTINGS.DATA_PATH / "kth_actions"
     CLASSES = ['boxing', 'handclapping', 'handwaving', 'walking', 'running', 'jogging']  #: The different scenarios that constitue this dataset.
     SHORT_CLASSES = ['walking', 'running', 'jogging']  #: Those scenarios where the sequence length might drop below the required frame count.
     MIN_SEQ_LEN = 30
     ACTION_SIZE = 0
     DATASET_FRAME_SHAPE = (64, 64, 3)
 
+    first_frame_rng_seed = 1234  #: Seed value for the random number generator used to determine the first frame out of a bigger sequence.
+
     def __init__(self, split, **dataset_kwargs):
         super(KTHActionsDataset, self).__init__(split, **dataset_kwargs)
         self.NON_CONFIG_VARS.extend(["data"])
@@ -64,7 +66,10 @@ class KTHActionsDataset(VPDataset):
         c, vid, seq = self.get_from_idx(i)
         dname = os.path.join(self.data_dir, c, vid[b'vid'].decode('utf-8'))
         frames = np.zeros((self.seq_len, *self.DATASET_FRAME_SHAPE))
-        first_frame = 0 if len(seq) <= self.seq_len else random.randint(0, len(seq) - self.seq_len)
+        if len(seq) <= self.seq_len:
+            first_frame = 0
+        else:
+            first_frame = random.Random(self.first_frame_rng_seed).randint(0, len(seq) - self.seq_len)
         last_frame = len(seq) - 1 if len(seq) <= self.seq_len else first_frame + self.seq_len - 1
         for i in range(first_frame, last_frame + 1):
             fname = os.path.join(dname, seq[i].decode('utf-8'))
@@ -75,7 +80,7 @@ class KTHActionsDataset(VPDataset):
         rgb = self.preprocess(np.array(frames))  # [t, c, h, w]
         actions = torch.zeros((self.total_frames, 1))  # [t, a], actions should be disregarded in training logic
 
-        data = { "frames": rgb, "actions": actions }
+        data = {"frames": rgb, "actions": actions, "origin": f"{dname}, start frame: {first_frame}"}
         return data
 
     def __len__(self):
@@ -84,7 +89,7 @@ class KTHActionsDataset(VPDataset):
     @classmethod
     def download_and_prepare_dataset(cls):
         from vp_suite.utils.utils import run_shell_command
-        import vp_suite.constants as constants
-        get_kth_command = f"{(constants.PKG_RESOURCES / 'get_dataset_kth.sh').resolve()} " \
+        from vp_suite.defaults import SETTINGS
+        get_kth_command = f"{(SETTINGS.PKG_RESOURCES / 'get_dataset_kth.sh').resolve()} " \
                           f"{str(cls.DEFAULT_DATA_DIR.resolve())}"
         run_shell_command(get_kth_command)
