@@ -4,8 +4,8 @@ import torch
 from torchvision.io import read_video
 from pathlib import Path
 
-from vp_suite.base.base_dataset import VPDataset, VPData
-import vp_suite.constants as constants
+from vp_suite.base import VPDataset, VPData
+from vp_suite.defaults import SETTINGS
 from vp_suite.utils.utils import set_from_kwarg, read_video
 
 
@@ -21,7 +21,7 @@ class Physics101Dataset(VPDataset):
     NAME = "Physics 101"
     REFERENCE = "http://phys101.csail.mit.edu/"
     IS_DOWNLOADABLE = "Yes"
-    DEFAULT_DATA_DIR = constants.DATA_PATH / "phys101"
+    DEFAULT_DATA_DIR = SETTINGS.DATA_PATH / "phys101"
     AVAILABLE_CAMERAS = ["Camera_1", "Camera_2", "Kinect_RGB_1"]  #: Available cameras/image sources.
     AVAILABLE_SUBSEQ = ["start", "middle", "end"]  #: Available (sub-)sequence extraction position identifiers.
     MIN_SEQ_LEN = 16
@@ -34,14 +34,8 @@ class Physics101Dataset(VPDataset):
     trainval_test_seed = 1612 #: The random seed used to separate training/validation and testing data. Value from the 'Noether Networks' code
 
     def __init__(self, split, **dataset_kwargs):
-        r"""
-
-        Args:
-            split ():
-            **dataset_kwargs ():
-        """
         super(Physics101Dataset, self).__init__(split, **dataset_kwargs)
-        self.NON_CONFIG_VARS.extend(["AVAILABLE_CAMERAS", "AVAILABLE_SUBSEQ", "vid_filepaths"])
+        self.NON_CONFIG_VARS.extend(["vid_filepaths"])
 
         # set attributes
         set_from_kwarg(self, dataset_kwargs, "camera", choices=self.AVAILABLE_CAMERAS)
@@ -50,8 +44,8 @@ class Physics101Dataset(VPDataset):
 
         # get video filepaths for train/val or test
         self.vid_filepaths: [Path] = sorted(list(Path(self.data_dir).rglob(f"**/{self.camera}.mp4")))
-        random.Random(self.trainval_test_seed).shuffle(self.vid_filepaths)
         slice_idx = int(len(self.vid_filepaths) * self.trainval_to_test_ratio)
+        random.Random(self.trainval_test_seed).shuffle(self.vid_filepaths)
         if self.split == "train":
             self.vid_filepaths = self.vid_filepaths[:slice_idx]
         else:
@@ -59,7 +53,8 @@ class Physics101Dataset(VPDataset):
 
     def __getitem__(self, i) -> VPData:
         # loaded video shape: [T, h, w, c], sitting in index 0 of the object returned by read_video()
-        vid = read_video(self.vid_filepaths[i], num_frames=self.total_frames)  # [T, h, w, c]
+        vid_fp = self.vid_filepaths[i]
+        vid = read_video(vid_fp, num_frames=self.total_frames)  # [T, h, w, c]
         if self.seq_step > 1:
             vid = vid[::self.seq_step]  # [t, h, w, c]
 
@@ -74,7 +69,7 @@ class Physics101Dataset(VPDataset):
         vid = self.preprocess(vid)
         actions = torch.zeros((self.total_frames, 1))  # [t, a], actions should be disregarded in training logic
 
-        data = { "frames": vid, "actions": actions }
+        data = {"frames": vid, "actions": actions, "origin": f"{vid_fp}, subseq mode: {self.subseq}"}
         return data
 
     def __len__(self):
