@@ -3,19 +3,19 @@ import torch
 import torch.nn as nn
 from vp_suite.model_blocks import SpatioTemporalLSTMCell as STCell,\
     ActionConditionalSpatioTemporalLSTMCell as ACSTCell
-from vp_suite.base.base_model import VideoPredictionModel
+from vp_suite.base import VPModel
 import torch.nn.functional as F
 from tqdm import tqdm
 
 
-class PredRNN_V2(VideoPredictionModel):
+class PredRNN_V2(VPModel):
     r"""
     This is a reimplementation of the model "PredRNN-V2", as introduced in "PredRNN: A Recurrent Neural Network for
     Spatiotemporal Predictive Learning" by Wang et al. (https://arxiv.org/pdf/2103.09504.pdf). This implementation
     is based on the official PyTorch implementation on https://github.com/thuml/predrnn-pytorch.
 
     PredRNN-V aims at learning partly disentangled spatial/temporal dynamics of the input domain and use this to
-    render more accurate predicted frames. The "Spatio-Temporal LSTM Cell" forms the heart of this model.
+    render more accurate predicted frames. The "Spatio-Temporal LSTM Cell" (ST cell) forms the heart of this model.
 
     Note:
         This model will use the whole frame sequence as an input, including the frames to be predicted. If you do not
@@ -24,37 +24,34 @@ class PredRNN_V2(VideoPredictionModel):
         if using the action-conditional variant, reverse scheduled sampling as well as 'conv_on_input'
         has to be set to 1/True!
     """
-
-    # model-specific constants
     NAME = "PredRNN++"
-    PAPER_REFERENCE = "https://arxiv.org/abs/2103.09504"  #: The paper where this model was introduced first.
-    CODE_REFERENCE = "https://github.com/thuml/predrnn-pytorch"  #: The code location of the reference implementation.
-    MATCHES_REFERENCE: str = "Yes"  #: A comment indicating whether the implementation in this package matches the reference.
+    PAPER_REFERENCE = "https://arxiv.org/abs/2103.09504"
+    CODE_REFERENCE = "https://github.com/thuml/predrnn-pytorch"
+    MATCHES_REFERENCE: str = "Yes"
     CAN_HANDLE_ACTIONS = False
     NEEDS_COMPLETE_INPUT = True
 
-    patch_size = 4
-    num_layers = 3
-    num_hidden = [128, 128, 128, 128]
-    filter_size = 5
-    stride = 1  #: stride for ST Cell
-    layer_norm: bool = False
-    reverse_input: bool = True
-    reconstruction_loss_scale = 0.1
-    decoupling_loss_scale = 100.0
-    inflated_action_dim = 3
+    patch_size = 4  #: During encoding, the image is sliced into patches of this size (height and width)
+    num_layers = 3  #: Number of ST Cell layers
+    num_hidden = [128, 128, 128, 128]  #: Hidden layer dimensionality per ST cell layer
+    filter_size = 5  #: Kernel size for ST cell and action-conditional convs
+    stride = 1  #: Stride for ST cell
+    inflated_action_dim = 3  #: Dimensionality of the 'inflated actions' (actions that have been transformed to tensors)
+    layer_norm: bool = False  #: Whether to use layer normalization in the ST cells
+    conv_actions_on_input: bool = True  #: Whether to convolve actions directly on the input
+    residual_on_action_conv: bool = True  #: Whether to use residual connections for the direct action convolution
 
-    scheduled_sampling: bool = True
-    sampling_stop_iter: int = 50000
-    sampling_changing_rate = 2e-5
-    reverse_scheduled_sampling: bool = False
-    r_sampling_step_1: int = 25000
-    r_sampling_step_2: int = 50000
-    r_exp_alpha: int = 5000
-    training_iteration: int = None
-    sampling_eta: float = None
-    conv_actions_on_input: bool = True
-    residual_on_action_conv: bool = True
+    reverse_input: bool = True  #: Whether to also train on the reversed version of training sequences
+    decoupling_loss_scale = 100.0  #: The scaling factor for the decoupling loss
+    scheduled_sampling: bool = True  #: Whether to use scheduled sampling during training
+    sampling_stop_iter: int = 50000  #: At which iteration to stop the scheduled sampling
+    sampling_changing_rate = 2e-5  #: Per-iteration changing rate for scheduled sampling
+    reverse_scheduled_sampling: bool = False  #: Whether to use reverse scheduled sampling
+    r_sampling_step_1: int = 25000  #: At which iteration to proceed to second reverse scheduled sampling phase
+    r_sampling_step_2: int = 50000  #: At which iteration to proceed to third reverse scheduled sampling phase
+    r_exp_alpha: int = 5000  #: Reverse scheduled sampling rate change regulator factor
+    training_iteration: int = None  #: Current number of training iteration (~how many training inferences were done so far)
+    sampling_eta: float = None  #: Sampling rate
 
     def __init__(self, device, **model_kwargs):
         super(PredRNN_V2, self).__init__(device, **model_kwargs)
@@ -167,24 +164,17 @@ class PredRNN_V2(VideoPredictionModel):
 
         memory = torch.zeros([b, self.num_hidden[0], self.rnn_h, self.rnn_w], device=self.device)
         mask_true = self._scheduled_sampling(b, context_frames, pred_frames, train)
+        first_t_with_blending = 1 if self.reverse_scheduled_sampling else context_frames
         x_gen = None
 
         # actual fwd pass
         for t in range(total_frames - 1):
-            if self.reverse_scheduled_sampling:
-                # reverse schedule sampling
-                if t == 0:
-                    net = x_patch[:, t]
-                else:
-                    net = mask_true[:, t - 1] * x_patch[:, t] \
-                          + (1 - mask_true[:, t - 1]) * x_gen
+            if t < first_t_with_blending:
+                net = x_patch[:, t]
             else:
-                # schedule sampling
-                if t < context_frames:
-                    net = x_patch[:, t]
-                else:
-                    net = mask_true[:, t - context_frames] * x_patch[:, t] \
-                          + (1 - mask_true[:, t - context_frames]) * x_gen
+                mask_ = mask_true[:, t - first_t_with_blending]
+                net = mask_ * x_patch[:, t] + (1 - mask_) * x_gen
+
             if self.action_conditional:
                 action = a_patch[:, t]
 
@@ -234,10 +224,10 @@ class PredRNN_V2(VideoPredictionModel):
             next_frames.append(x_gen)
 
         # finalize
-        predictions_patch = torch.stack(next_frames, dim=1)  # [b, t', cpp, h_, w_]
-        predictions = self._reshape_patch_back(predictions_patch)[:, -pred_frames:]  # [b, t_pred, c, h, w]
+        predictions_patch = torch.stack(next_frames[-pred_frames:], dim=1)  # [b, t_pred, cpp, h_, w_]
+        predictions = self._reshape_patch_back(predictions_patch)  # [b, t_pred, c, h, w]
         decouple_loss = torch.mean(torch.stack(decouple_loss, dim=0))
-        return predictions, {"ST-LSTM decouple loss": decouple_loss}
+        return predictions, {"ST-LSTM decouple loss": self.decoupling_loss_scale * decouple_loss}
 
     def _reshape_patch(self, x):
         b, t, c, h, w = x.shape
