@@ -1,7 +1,8 @@
-import random, json, os
+import random, json, os, time
 import warnings
 from typing import List, Dict, Any
 from pathlib import Path
+from copy import deepcopy
 
 import numpy as np
 import torch
@@ -10,39 +11,37 @@ from torch.utils.data import DataLoader
 import wandb
 from tqdm import tqdm
 
-import vp_suite.constants as constants
-from vp_suite.utils.dataset_wrapper import DatasetWrapper
+from vp_suite.defaults import SETTINGS, DEFAULT_RUN_CONFIG
+from vp_suite.utils.dataset_wrapper import VPDatasetWrapper
 from vp_suite.datasets import DATASET_CLASSES
-from vp_suite.base.base_model import VideoPredictionModel
+from vp_suite.base import VPModel
 from vp_suite.models import MODEL_CLASSES, AVAILABLE_MODELS
 from vp_suite.models.copy_last_frame import CopyLastFrame
 from vp_suite.measure import LOSS_CLASSES
 from vp_suite.measure.loss_provider import PredictionLossProvider
 from vp_suite.measure.metric_provider import PredictionMetricProvider
-from vp_suite.utils.visualization import visualize_vid
+from vp_suite.utils.visualization import visualize_vid, visualize_sequences
 from vp_suite.utils.utils import timestamp
 from vp_suite.utils.compatibility import check_model_and_data_compat, check_run_and_model_compat
 
 
 class VPSuite:
-    r"""TODO.
-
-    TODO.
+    r"""
+    This class is the main workbench of `vp-suite`, in the sense that upon instantiating this class you've got access to
+    all the functionalities of `vp-suite`: Loading and using datasets, loading or instantiating new models and training
+    or testing them on the loaded datasets.
 
     Attributes:
-        device(str): A string specifying which device to work on ('cuda' or 'gpu')
-        datasets(List[:class:`DatasetWrapper`]): The loaded datasets.
-        models(List[:class:`VideoPredictionModel`]): The loaded/created models.
-
+        device(str): A string specifying which device to work on ('cuda' for GPU usage or 'cpu' for CPU).
+        datasets(List[:class:`VPDatasetWrapper`]): The loaded datasets.
+        models(List[:class:`VPModel`]): The loaded/created models.
     """
-
-    _DEFAULT_RUN_CONFIG : Path = constants.PKG_RESOURCES / 'run_config.json'
-
-    def __init__(self, device="cuda"):
+    def __init__(self, device: str = "cuda"):
         r"""
+        Instantiates the `VPSuite`, setting device and clearing loaded models and datasets.
 
         Args:
-            device ():
+            device (str): A string identifying whether to use GPU or CPU.
         """
         self.device = "cuda" if device == "cuda" and torch.cuda.is_available() else "cpu"
 
@@ -52,91 +51,106 @@ class VPSuite:
     @property
     def training_sets(self):
         r"""
-
-        Returns:
-
+        Returns: A list of all loaded training sets (last one in list is last loaded training set).
         """
         return [d for d in self.datasets if d.is_training_set()]
 
     @property
     def test_sets(self):
         r"""
-
-        Returns:
-
+        Returns: A list of all loaded test sets (last one in list is last loaded test set).
         """
         return [d for d in self.datasets if d.is_test_set()]
 
     def clear_datasets(self):
         r"""
-
-        Returns:
-
+        Empties the list of loaded datasets.
         """
-        self.datasets : List[DatasetWrapper] = []
+        self.datasets : List[VPDatasetWrapper] = []
 
     def clear_models(self):
         r"""
-
-        Returns:
-
+        Empties the list of loaded models.
         """
-        self.models : List[VideoPredictionModel] = []
+        self.models : List[VPModel] = []
 
-    def load_dataset(self, dataset, split="train", **dataset_kwargs):
+    def load_dataset(self, dataset_id: str, split: str = "train", **dataset_kwargs):
         r"""
-
+        Creates the dataset specified by given dataset id and appends it to `VPSuite`'s list of loaded datasets.
+        
         Args:
-            dataset ():
-            split ():
-            value_min ():
-            value_max ():
-            **dataset_kwargs ():
-
-        Returns:
-
+            dataset_id (str): The string ID mapping of the desired dataset.
+            split (str): This string specifies whether to load the dataset in training or testing mode.
+            **dataset_kwargs (Any): Optional additional dataset configuration options.
         """
-        dataset_class = DATASET_CLASSES[dataset]
-        dataset = DatasetWrapper(dataset_class, split, **dataset_kwargs)
+        # create dataset wrapper
+        dataset_class = DATASET_CLASSES[dataset_id]
+        dataset = VPDatasetWrapper(dataset_class, split, **dataset_kwargs)
         print(f"loaded dataset '{dataset.NAME}' from {dataset.data_dir} "
               f"(action size: {dataset.action_size})")
+
+        # if seq information is specified, directly execute set_seq_len
+        if any([k in dataset_kwargs.keys() for k in ["context_frames", "pred_frames", "seq_step"]]):
+            context_frames = dataset_kwargs.pop("context_frames", DEFAULT_RUN_CONFIG["context_frames"])
+            pred_frames = dataset_kwargs.pop("pred_frames", DEFAULT_RUN_CONFIG["pred_frames"])
+            seq_step = dataset_kwargs.pop("seq_step", DEFAULT_RUN_CONFIG["seq_step"])
+            dataset.set_seq_len(context_frames, pred_frames, seq_step)
+
         self.datasets.append(dataset)
 
-    def download_dataset(self, dataset):
-        dataset_class = DATASET_CLASSES[dataset]
-        dataset_class.download_and_prepare_dataset()
-
-    def load_model(self, model_dir, ckpt_name="best_model.pth"):
+    def download_dataset(self, dataset_id: str):
         r"""
+        Downloads the dataset specified by given dataset ID.
 
         Args:
-            model_dir ():
-            ckpt_name ():
+            dataset_id (str): The string ID mapping of the desired dataset.
+        """
+        dataset_class = DATASET_CLASSES[dataset_id]
+        dataset_class.download_and_prepare_dataset()
 
-        Returns:
+    def list_available_datasets(self):
+        r"""
+        Prints a list of all available datasets and their corresponding string IDs.
+        """
+        for dataset_id, dataset_class in DATASET_CLASSES.items():
+            print(f"'{dataset_id}': {dataset_class.NAME}")
 
+    def list_available_models(self):
+        r"""
+        Prints a list of all available models and their corresponding string IDs.
+        """
+        for model_id, model_class in MODEL_CLASSES.items():
+            print(f"'{model_id}': {model_class.NAME}")
+
+    def load_model(self, model_dir: str, ckpt_name: str = "best_model.pth"):
+        r"""
+        Loads the model saved in the specified checkpoint file or the specified directory
+         and appends it to `VPSuite`'s list of loaded models.
+
+        Args:
+            model_dir (str): Relative path to the directory containing the saved model.
+            ckpt_name (str): File name of the saved model.
         """
         model_ckpt = os.path.join(model_dir, ckpt_name)
         model = torch.load(model_ckpt)
+        model.model_dir = model_dir
         self._model_setup(model, loaded=True)
 
-    def create_model(self, model_type, action_conditional=False, **model_kwargs):
+    def create_model(self, model_id: str, action_conditional: bool = False, **model_kwargs):
         r"""
+        Creates the model specified by given string ID and appends it to `VPSuite`'s list of loaded models.
 
         Args:
-            model_type ():
-            action_conditional ():
-            **model_kwargs ():
-
-        Returns:
-
+            model_id (str): The string ID corresponding to your desired model.
+            action_conditional (bool): If the model supports actions, this variable determines whether the model will actually use provided actions for prediction.
+            **model_kwargs (Any): Optional additional model configuration options.
         """
 
         # parameter processing
-        if model_type not in AVAILABLE_MODELS:
+        if model_id not in AVAILABLE_MODELS:
             raise ValueError(f"invalid model type specified! Available model types: {AVAILABLE_MODELS}")
 
-        model_class = MODEL_CLASSES[model_type]
+        model_class = MODEL_CLASSES[model_id]
         for param in model_class.REQUIRED_ARGS:
             if param not in model_kwargs.keys():
                 print(f"model parameter '{param}' not specified -> trying to take from last loaded dataset...")
@@ -156,15 +170,13 @@ class VPSuite:
         model = model_class(self.device, **model_kwargs).to(self.device)
         self._model_setup(model)
 
-    def _model_setup(self, model, loaded=False):
+    def _model_setup(self, model: VPModel, loaded: bool = False):
         r"""
+        Internal model setup, also appending the model to the list of loaded models.
 
         Args:
-            model ():
-            loaded ():
-
-        Returns:
-
+            model (VPModel): The video prediction model.
+            loaded (bool): Identifies whether the model has been loaded (=True) or newly created (=False).
         """
         ac_str = "(action-conditional)" if model.config["action_conditional"] else ""
         loaded_str = "loaded" if loaded else "created new"
@@ -174,15 +186,13 @@ class VPSuite:
         print(f" - Model parameters (total / trainable): {total_params} / {trainable_params}")
         self.models.append(model)
 
-    def _prepare_run(self, split="train", **run_args):
+    def _prepare_run(self, split: str = "train", **run_kwargs):
         r"""
+        Prepares the upcoming run by fetching/setting configuration, models and datasets.
 
         Args:
-            split ():
-            **run_args ():
-
-        Returns:
-
+            split (str): Determines whether this run is going to be a training/hyperopt run or a test run.
+            **run_kwargs (Any): Optional specified run configuration parameters (will override the defaults).
         """
         if len(self.models) == 0:
             raise RuntimeError("No model available. Load a pretrained model "
@@ -193,18 +203,14 @@ class VPSuite:
         elif split == "test" and len(self.test_sets) == 0:
             raise ValueError("No test sets loaded. Load a dataset in test mode "
                              "before starting training or test runs")
-        with open(str(self._DEFAULT_RUN_CONFIG.resolve()), 'r') as tc_file:
-            run_config = json.load(tc_file)
+        run_config = deepcopy(DEFAULT_RUN_CONFIG)
 
         # update config
-        if not all([run_arg in run_config.keys() for run_arg in run_args.keys()]):
+        if not all([run_arg in run_config.keys() for run_arg in run_kwargs.keys()]):
             raise ValueError(f"Only the following run arguments are supported: {run_config.keys()}")
-        run_config.update(run_args)
+        run_config.update(run_kwargs)
 
-        # seed
-        random.seed(run_config["seed"])
-        np.random.seed(run_config["seed"])
-        torch.manual_seed(run_config["seed"])
+        self._set_seeds(run_config["seed"])
 
         # opt. direction
         run_config["opt_direction"] = "maximize" if LOSS_CLASSES[run_config["val_rec_criterion"]].BIGGER_IS_BETTER \
@@ -212,22 +218,46 @@ class VPSuite:
 
         return run_config
 
+    def _set_seeds(self, seed: int):
+        r"""
+        Sets the general rng seeds for Python, numpy and PyTorch.
+
+        Args:
+            seed (int): The random seed for the rng's.
+
+        Warning: This should remain the only code location where the general rng seeds are set!
+        """
+        random.seed(seed)
+        np.random.seed(seed)
+        torch.manual_seed(seed)
+
+    def reset_rng(self, seed: int):
+        r"""
+        Resets the RNG for all datasets and rng.
+
+        Args:
+            seed (int): The random seed for the rng's.
+        """
+        self._set_seeds(seed)
+        for dataset in self.datasets:
+            dataset.reset_rng()
+
 # ===== TRAINING ================================================================
 
-    def _prepare_training(self, dataset_idx, model_idx, **training_kwargs):
+    def _prepare_training(self, dataset_idx: int, model_idx: int, **run_kwargs):
         r"""
         Prepares a single dataset and a single model for training.
+
         Args:
-            **training_kwargs ():
-
-        Returns:
-
+            dataset_idx (int): The list index of the dataset that should be used for training.
+            model_idx (int) The list index of the model that should be trained on.
+            **run_kwargs (Any): Optional specified run configuration parameters (will override the defaults).
         """
-        run_config = self._prepare_run("train", **training_kwargs)
+        run_config = self._prepare_run("train", **run_kwargs)
 
         try:
-            dataset : DatasetWrapper = self.training_sets[dataset_idx]
-            model : VideoPredictionModel = self.models[model_idx]
+            dataset: VPDatasetWrapper = self.training_sets[dataset_idx]
+            model: VPModel = self.models[model_idx]
         except IndexError:
             raise ValueError("given indices for model and/or dataset are invalid")
 
@@ -241,29 +271,47 @@ class VPSuite:
 
         return model, dataset, run_config
 
-    def train(self, trial=None, dataset_idx=-1, model_idx=-1, **training_kwargs):
+    def train(self, trial=None, dataset_idx: int = -1, model_idx: int = -1, **run_kwargs):
         r"""
+        Executes a training run on a single model and a single dataset.
+        After preparation, the main training loops run training epochs until the max number of epochs
+        or the time limit is reached.
+
+        During each epoch: 1. the whole dataset is iterated through in (model-specific)
+        training iterations that employ forward and backward passes. 2. Each model is validated on the whole validation
+        set and saved if it improved its performance on that set. 3. Every few epochs, prediction visualizations are
+        created and saved to the disk. 4. Current model performance is logged.
 
         Args:
-            trial ():
-            **training_kwargs ():
-
-        Returns:
+            trial (Any): If calling this function within a hyperparameter optimization run, this object cantains the necessary parameters. Otherwise, it's None.
+            dataset_idx (int): The list index of the dataset that should be used for training.
+            model_idx (int) The list index of the model that should be trained on.
+            **run_kwargs (Any): Optional specified run configuration parameters (will override the defaults).
 
+        Returns: The best obtained validation loss (the corresponding model is saved as 'best_model.pt').
         """
         # PREPARATION
-        model, dataset, run_config = self._prepare_training(dataset_idx, model_idx, **training_kwargs)
+        model, dataset, run_config = self._prepare_training(dataset_idx, model_idx, **run_kwargs)
         train_data, val_data = dataset.train_data, dataset.val_data
         train_loader = DataLoader(train_data, batch_size=run_config["batch_size"], shuffle=True, num_workers=4,
                                   drop_last=True)
         val_loader = DataLoader(val_data, batch_size=1, shuffle=False, num_workers=0, drop_last=True)
         best_val_loss = float("inf")
-        out_path = Path(run_config["out_dir"]) if run_config["out_dir"] is not None \
-            else constants.OUT_PATH / timestamp('train')
-        out_path.mkdir(parents=True, exist_ok=True)
-        model.model_dir = str(out_path.resolve())
+
+        # re-use model_dir of pre-loaded/pre-initialized models if no out_dir has been specified
+        if run_config["out_dir"] is None and model.model_dir is not None:
+            print(f"Using existing model save location ({model.model_dir})...")
+            out_path = Path(model.model_dir)
+        else:
+            out_dir = run_config["out_dir"] or SETTINGS.OUT_PATH / timestamp('train')  # fetch default if None
+            out_path = Path(out_dir)
+            out_path.mkdir(parents=True, exist_ok=True)
+            model.model_dir = str(out_path.resolve())
+
         best_model_path = str((out_path / 'best_model.pth').resolve())
         with_training = model.TRAINABLE and not run_config["no_train"]
+        with_validation = not run_config["no_val"]
+        with_wandb = not run_config["no_wandb"]
 
         # HYPERPARAMETER OPTIMIZATION
         optuna_config = run_config.get("optuna", None)
@@ -285,7 +333,8 @@ class VPSuite:
                         run_config[param] = suggest(param, p_dict["min"], p_dict["max"], step=step)
 
         # assemble and save combined configuration
-        config : Dict[str, Any] = {**run_config, **model.config, **dataset.config, "device": self.device}
+        config: Dict[str, Any] = {**run_config, **model.config, **dataset.config, "device": self.device,
+                                  "model_name": model.NAME, "dataset_name": dataset.NAME}
         save_config = {"run": run_config, "model": model.config,
                        "dataset": dataset.config, "device": self.device}
         with open(str((out_path / 'run_cfg.json').resolve()), "w") as cfg_file:
@@ -293,10 +342,10 @@ class VPSuite:
                       default=lambda o: str(o) if callable(getattr(o, "__str__", None)) else '<not serializable>')
 
         # WandB
-        if not run_config["no_wandb"]:
+        if with_wandb:
             wandb_reinit = using_optuna and trial.number > 0
             wandb.init(config=config, project="vp-suite-training",
-                       dir=str(constants.WANDB_PATH.resolve()), reinit=wandb_reinit)
+                       dir=str(SETTINGS.WANDB_PATH.resolve()), reinit=wandb_reinit)
 
         # OPTIMIZER
         optimizer, optimizer_scheduler = None, None
@@ -316,65 +365,80 @@ class VPSuite:
             def loss_improved(cur_loss, best_loss): return cur_loss < best_loss
 
         # --- MAIN LOOP ---
+        training_timeout = time.time() + config["max_training_hours"] * 3600
         for epoch in range(0, run_config["epochs"]):
+            print(f"\nEpoch: {epoch+1} of {config['epochs']}")
 
             # train
-            print(f'\nTraining (epoch: {epoch+1} of {config["epochs"]})')
             if with_training:
+                print("Training...")
                 model.train_iter(config, train_loader, optimizer, loss_provider, epoch)
             else:
                 print("Skipping training loop.")
 
             # eval
-            print("Validating...")
-            val_losses, indicator_loss = model.eval_iter(config, val_loader, loss_provider)
-            if with_training:
-                optimizer_scheduler.step(indicator_loss)
-            print("Validation losses (mean over entire validation set):")
-            for k, v in val_losses.items():
-                print(f" - {k}: {v}")
+            val_losses = dict()
+            if with_validation:
+                print("Validating...")
+                val_losses, indicator_loss = model.eval_iter(config, val_loader, loss_provider)
+                if with_training:
+                    optimizer_scheduler.step(indicator_loss)
+                print("Validation losses (mean over entire validation set):")
+                for k, v in val_losses.items():
+                    print(f" - {k}: {v}")
 
-            # save model if last epoch improved indicator loss
-            cur_val_loss = indicator_loss.item()
-            if loss_improved(cur_val_loss, best_val_loss):
-                best_val_loss = cur_val_loss
+                # save model if last epoch improved indicator loss
+                cur_val_loss = indicator_loss.item()
+                if loss_improved(cur_val_loss, best_val_loss):
+                    best_val_loss = cur_val_loss
+                    torch.save(model, best_model_path)
+                    print(f"Minimum indicator loss ({config['val_rec_criterion']}) reduced -> model saved!")
+            else:
+                print("Skipping validation loop and simply saving current model as the 'best' model.")
                 torch.save(model, best_model_path)
-                print(f"Minimum indicator loss ({config['val_rec_criterion']}) reduced -> model saved!")
 
             # visualize current model performance every nth epoch, using eval mode and validation data.
             if (epoch+1) % config["vis_every"] == 0 and not config["no_vis"]:
                 print("Saving visualizations...")
-                vis_out_path = out_path / f"vis_ep_{epoch+1:03d}"
-                vis_out_path.mkdir()
+                vis_out_dir = out_path / f"vis_ep_{epoch+1:03d}"
+                vis_out_dir.mkdir(exist_ok=True)  # overrides existing visualizations (e.g. from previous runs)
+                vis_idx = np.random.choice(len(val_data), config["n_vis"], replace=False)
                 visualize_vid(val_data, config["context_frames"], config["pred_frames"], model,
-                              config["device"], vis_out_path, num_vis=10)
+                              config["device"], vis_out_dir, vis_idx, config["vis_mode"])
 
-                if not config["no_wandb"]:
-                    vid_filenames = sorted(os.listdir(str(vis_out_path)))
-                    log_vids = {fn: wandb.Video(str(vis_out_path / fn), fps=4, format=fn.split(".")[-1])
+                if with_wandb:
+                    vid_filenames = sorted(os.listdir(str(vis_out_dir)))
+                    log_vids = {fn: wandb.Video(str(vis_out_dir / fn), fps=4, format=fn.split(".")[-1])
                                 for i, fn in enumerate(vid_filenames)}
                     wandb.log(log_vids, commit=False)
 
             # final bookkeeping
-            if not config["no_wandb"]:
+            if with_validation and with_wandb:
                 wandb.log(val_losses, commit=True)
+            if time.time() > training_timeout:
+                print("Maximum training time exceeded, leaving training loop...")
+                break
 
-        # finishing
+        # finishing training by saving final model and returning best performance on validation set
         print("\nTraining done, cleaning up...")
         torch.save(model, str((out_path / 'final_model.pth').resolve()))
         wandb.finish()
         return best_val_loss  # return best validation loss for hyperparameter optimization
 
-    def hyperopt(self, optuna_config, n_trials=30, dataset_idx=-1, model_idx=-1, **run_kwargs):
+    def hyperopt(self, optuna_config: dict, n_trials: int = 30, dataset_idx: int = -1, model_idx: int = -1,
+                 **run_kwargs):
         r"""
+        Executes a hyperparameter optimization process using the bayesian optimization framework optuna.
+        A pre-defined number of training runs ("trials") are executed sequentially on the same model and datasets
+        under varying (hyperparameter) configurations. After the optimization, the best configuration is printed to the
+        terminal.
 
         Args:
-            optuna_config ():
-            n_trials ():
-            **run_kwargs ():
-
-        Returns:
-
+            optuna_config (dict): Optuna run configuration, specifies the search space (parameters and their search ranges) of the optimization.
+            n_trials (int): The number of training runs executed within the optimization.
+            dataset_idx (int): The list index of the dataset that should be used for training.
+            model_idx (int) The list index of the model that should be trained on.
+            **run_kwargs (Any): Optional specified run configuration parameters (will override the defaults). Apply to all runs within the optimizaion.
         """
         from functools import partial
         from vp_suite.utils.utils import check_optuna_config
@@ -396,17 +460,15 @@ class VPSuite:
 
     def _prepare_testing(self, **run_kwargs):
         r"""
-        Prepares multiple datasets and models for testing
+        Prepares all loaded datasets and models for testing
+
         Args:
-            **run_kwargs ():
-
-        Returns:
-
+            **run_kwargs (Any): Optional specified run configuration parameters (will override the defaults).
         """
         run_config = self._prepare_run("test", **run_kwargs)
 
         # prepare datasets for testing
-        test_sets : List[DatasetWrapper] = self.test_sets
+        test_sets: List[VPDatasetWrapper] = self.test_sets
         for test_set in test_sets:
             test_set.set_seq_len(run_config["context_frames"], run_config["pred_frames"], run_config["seq_step"])
             assert test_set.is_ready, "test set is not ready even though set_seq_len has just been called"
@@ -441,30 +503,32 @@ class VPSuite:
         test_sets_and_model_lists = zip(test_sets, model_lists_all_test_sets)
         return test_sets_and_model_lists, run_config
 
-    def _test_on_dataset(self, model_info_list, dataset, run_config, brief_test):
+    def _test_on_dataset(self, model_info_list, dataset: VPDatasetWrapper, run_config: dict, brief_test: bool):
         r"""
+        Tests all models on a single dataset. Iterates through the whole dataset, retrieving the predictions of all
+        specified models on each data point and calculating pre-specified performance metrics.
 
         Args:
-            model_info_list ():
-            dataset ():
-            run_config ():
-            brief_test ():
-
-        Returns:
-
+            model_info_list (Any): A list containing the models to be tested as well as other needed information/objects such as adapters.
+            dataset (VPDatasetWrapper): The dataset to be tested on.
+            run_config (dict): The run configuration used for testing. For all unspecified run configuration parameters, the default configuration is used.
+            brief_test (bool): If specified, only a brief sample check is done instead of fully iterating over the test set.
         """
         # PREPARATION
         test_data = dataset.test_data
-        test_loader = DataLoader(test_data, batch_size=1, shuffle=True, num_workers=0)
+        test_loader = DataLoader(test_data, batch_size=1, shuffle=False, num_workers=0)
         if len(test_loader) < 1:
             raise RuntimeError("loaded dataset does not contain any data (len < 1)")
         test_mode = "brief" if brief_test else "full"
         eval_length = min(len(test_loader), 10) if test_mode == "brief" else len(test_loader)
 
         # assemble and save combined configuration
-        config = {**run_config, **dataset.config, "device": self.device}
+        config: Dict[str, Any] = {**run_config, **dataset.config, "device": self.device, "dataset_name": dataset.NAME}
+        with_wandb = not config["no_wandb"]
 
         # evaluation / metric calc.
+        context_frames = config["context_frames"]
+        pred_frames = config["pred_frames"]
         iter_loader = iter(test_loader)
         with torch.no_grad():
             metric_provider = PredictionMetricProvider(config)
@@ -475,33 +539,40 @@ class VPSuite:
                 for (model, preprocess, postprocess, model_metrics_per_dp) in model_info_list:
                     input, target, actions = model.unpack_data(data, config)
                     input = preprocess(input)  # test format to model format
+                    model.eval()
                     if getattr(model, "use_actions", False):
-                        pred, _ = model(input, pred_frames=config["pred_frames"], actions=actions)
+                        pred, _ = model(input, pred_frames=pred_frames, actions=actions)
                     else:
-                        pred, _ = model(input, pred_frames=config["pred_frames"])
+                        pred, _ = model(input, pred_frames=pred_frames)
+                    model.train()
                     pred = postprocess(pred)  # model format to test format
                     cur_metrics = metric_provider.get_metrics(pred, target, all_frame_cnts=True)
                     model_metrics_per_dp.append(cur_metrics)
 
         # save visualizations
+        timestamp_test = timestamp('test')
+        vis_out_dir = SETTINGS.OUT_PATH / timestamp_test
+        vis_out_dir.mkdir()
         if not config["no_vis"]:
             print(f"Saving visualizations for trained models...")
-            num_vis = 5
-            vis_idx = np.random.choice(len(test_data), num_vis, replace=False)
-            for i, (model, _, _, _) in enumerate(model_info_list):
-                if model.model_dir is None:
-                    continue  # don't print for models that don't have a run dir (i.e. baseline models)
-                vis_out_dir = Path(model.model_dir) / f"vis_{timestamp('test')}"
-                vis_out_dir.mkdir()
-                visualize_vid(test_data, config["context_frames"], config["pred_frames"],
-                              model, config["device"], vis_out_dir, vis_idx=vis_idx)
+            vis_idx = np.random.choice(len(test_data), config["n_vis"], replace=False)
+            if test_data.ON_THE_FLY:  # if data is generated on-the-fly, reset dataset's RNG because the loaders could have mingled with it
+                self.reset_rng(config["seed"])
+
+            models = [m_info[0] for m_info in model_info_list]
+            if config["vis_compare"]:
+                vis_context_frame_idx = config["vis_context_frame_idx"] or list(range(context_frames))
+            else:
+                vis_context_frame_idx = None
+            visualize_sequences(test_data, context_frames, pred_frames, models, config["device"],
+                                vis_out_dir, vis_idx, vis_context_frame_idx, config["vis_mode"])
 
         # log or display metrics
         if eval_length > 0:
             wandb_full_suffix = f"{test_mode} test"
             for i, (model, _, _, model_metrics_per_dp) in enumerate(model_info_list):
                 # model_metrics_per_dp: list of N lists of F metric dicts (called D).
-                # each D_{n, f} contains all calculated metrics for datapoint 'n' and a prediction horizon of 'f' frames.
+                # each D_{n,f} contains all calculated metrics for datapoint 'n' and a prediction horizon of 'f' frames.
                 # -> Aggregate these metrics over all n, keeping the specific metrics/prediction horizons separate
                 datapoint_range = range(len(model_metrics_per_dp))
                 frame_range = range(len(model_metrics_per_dp[0]))
@@ -514,19 +585,24 @@ class VPSuite:
                 ]
 
                 # Log model to WandB
-                if not config["no_wandb"]:
+                if with_wandb:
                     print("Logging test results to WandB for all models...")
                     wandb.init(config={"test_mode": test_mode, "model_dir": model.model_dir},
-                               project="vp-suite-testing", name=f"{model.NAME}{wandb_full_suffix}",
-                               dir=str(constants.WANDB_PATH.resolve()), reinit=(i > 0))
+                               project="vp-suite-testing", name=f"{model.NAME} ({wandb_full_suffix})",
+                               dir=str(SETTINGS.WANDB_PATH.resolve()), reinit=(i > 0))
                     for f, mean_metric_dict in enumerate(mean_metric_dicts):
                         wandb.log({"pred_frames": f+1, **mean_metric_dict})
+                    if not config["no_vis"] and model.model_dir is not None:
+                        vid_filenames = [fn for fn in sorted(os.listdir(str(vis_out_dir)))
+                                         if fn.split(".")[-1] in ["mp4", "gif"]]
+                        model_log_vids = {fn: wandb.Video(str(vis_out_dir / fn), fps=4, format=fn.split(".")[-1])
+                                          for i, fn in enumerate(vid_filenames)}
+                        wandb.log(model_log_vids)
                     if i == len(model_info_list) - 1:
                         wandb.finish()
 
                 # Per-model/per-pred.-horizon console log of the metrics
                 else:
-                    print("Printing out test results to terminal...")
                     print(f"\n{model.NAME} (path: {model.model_dir}): ")
                     for f, mean_metric_dict in enumerate(mean_metric_dicts):
                         print(f"pred_frames: {f + 1}")
@@ -535,13 +611,15 @@ class VPSuite:
 
     def test(self, brief_test=False, **run_kwargs):
         r"""
+        Tests all loaded models and datasets, one dataset after the other.
+        For each dataset test, the whole test set is iterated through and each
+        data point is fed into each model to obtain performance metrics.
+        After iteration, visualizations are created of the predictions of all models on randomly selected
+        frame sequences, and performance metrics of each model on each dataset are logged in a comparable way.
 
         Args:
-            brief_test ():
-            **run_kwargs ():
-
-        Returns:
-
+            brief_test (bool): If specified, only a brief sample check is done instead of fully iterating over the test set.
+            **run_kwargs (Any): Optional testing configuration.
         """
         test_sets_and_model_lists, run_config = self._prepare_testing(**run_kwargs)
         for test_set, model_info_list in test_sets_and_model_lists:
